""" Non-HDF5 utility functions """
import copy as _copy
from collections import OrderedDict as _OrderedDict

def filterlist(in_list, filters, keep_filtered_items=True, exclusive=True):
    """
    Parameters
    ----------
    in_list : list
        List of strings to filter

    filters : str, list, tuple
        Find filters (or entries of filters) in in_list

    keep_filtered_items : bool
        Returns entries from in_list that DO have filters (INCLUDE filter).
        If False, EXCLUDE filter

    exclusive : bool
        Filter is exclusive, i.e. includes/excludes in_list entries that
        have ALL filters. Otherwise, non-exclusive and any entry with A
        filter are excluded/included.

    Returns
    -------
        list : filtered list

    """
    if isinstance(filters, (tuple, list)):
        filter_list = filters
    elif isinstance(filters, str):
        filter_list = [filters]
    else:
        raise TypeError('filters must be of type str, tuple, or list')

    def condition(keep_it, item):
        """ Keep or don't keep item depending on keep_it bool """
        if keep_it:
            return item
        else:
            return not item

    if exclusive:
        out_list = _copy.deepcopy(in_list)
        for current_filt in filter_list:
            out_list = [entry for entry in out_list if condition(keep_filtered_items,
                                                                 entry.count(current_filt))]
    else:
        out_list = []
        for current_filt in filter_list:
            out_list.extend([entry for entry in in_list if condition(keep_filtered_items,
                                                                     entry.count(current_filt))])
            # Removes duplicates
            out_list = list(_OrderedDict.fromkeys(out_list))
    
    return out_list




""" Utility functions """
import h5py as _h5py

__all__ = ['FidOrFile', 'hdf_is_open']

class FidOrFile:
    """
    Class for opening an HDF5 file and returning a file ID (fid) or if passed
    and already open fid, passing it along (pass-thru). Primarily for enabling
    functions and methods to operate on open and closed files.

    Parameters
    ----------
    file : str or h5py.File
        Filename or File-object for open HDF5 file

    mode : str
        If opening a file, open with mode. Available: r,r+,w,w-,x,a

    Attributes
    ----------
    is_fid : bool
        Was the input file actually an fid.

    fid : h5py.File object
        File ID
    """
    def __init__(self, file=None, mode='r'):
        self.is_fid = None
        self.fid = None
        if file is not None:
            self.return_fid_from_file(file, mode=mode)

    def return_fid_from_file(self, file, mode='r'):
        """
        Return an open fid (h5py.File). If provided a string, open file, else
        pass-thru given fid.

        Parameters
        ----------
        file : str or h5py.File
            Filename or File-object for open HDF5 file

        mode : str
            If opening a file, open with mode. Available: r,r+,w,w-,x,a

        Returns
        -------
        fid : h5py.File object
            File ID

        """
        self.is_fid = isinstance(file, _h5py.File)
        if not self.is_fid:
            self.fid = _h5py.File(file, mode=mode)
        else:
            self.fid = file
        return self.fid

    def close_if_file_not_fid(self):
        """ Close the file if originally a filename (not a fid) was passed """
        if not self.is_fid:
            return self.fid.close()
        else:
            return None

def hdf_is_open(fid):
    """ Is an HDF file open via fid """
    status = fid.fid.valid
    if status == 0:
        return False
    elif status == 1:
        return True
    else:
        return None



""" Test HDF-related utilities """
import os

import pytest

import h5py
import numpy as np

from lazy5.utils import FidOrFile, hdf_is_open

@pytest.fixture(scope="module")
def hdf_dataset():
    """ Setups and tears down a sample HDF5 file """
    filename = 'temp_test_utils.h5'
    fid = h5py.File(filename, 'w')
    data_m, data_n, data_p = [20, 22, 24]
    data = np.random.randn(data_m, data_n, data_p)
    fid.create_dataset('base', data=data)

    yield filename, fid

    # Tear-down
    if hdf_is_open(fid):
        fid.close()
    os.remove(filename)

def test_fid_or_file_filename_provided(hdf_dataset):
    """ Test FidOrFile Class with provided filename """
    filename, fid = hdf_dataset

    fof = FidOrFile(filename)
    assert fof.fid.fid.valid == 1
    assert fof.fid is not None
    assert fof.is_fid == False

    fof.fid.close()

def test_fid_or_file_fid_provided(hdf_dataset):
    """ Test FidOrFile Class with provided fid """
    filename, fid = hdf_dataset

    fof = FidOrFile(fid)
    assert fof.fid.fid.valid == 1
    assert fof.fid is not None
    assert fof.is_fid == True

def test_fid_or_file_close_if_not_fid(hdf_dataset):
    """ Test close if filename was provided """
    filename, fid = hdf_dataset

    fof = FidOrFile(fid)
    fof.close_if_file_not_fid()
    assert fof.fid.fid.valid == 1

    fof = FidOrFile(filename)
    fof.close_if_file_not_fid()
    assert fof.fid.fid.valid == 0

def test_hdf_is_open(hdf_dataset):
    """ Test hdf_is_open function """
    filename, fid = hdf_dataset

    assert hdf_is_open(fid) == True
    fid.close()
    
    assert hdf_is_open(fid) == False




""" Test inspection of HDF5 files """
import os

import h5py
import pytest

import numpy as np

from lazy5.inspect import (get_groups, get_datasets, get_hierarchy,
                           get_attrs_dset)
from lazy5.utils import hdf_is_open

@pytest.fixture(scope="module")
def hdf_dataset():
    """ Setups and tears down a sample HDF5 file """
    filename = 'temp_test.h5'
    fid = h5py.File(filename, 'w')
    data_m, data_n, data_p = [20, 22, 24]
    data = np.random.randn(data_m, data_n, data_p)

    fid.create_dataset('base', data=data)

    grp1 = fid.create_group('Group1')
    grp3 = fid.create_group('Group2/Group3')
    grp6 = fid.create_group('Group4/Group5/Group6')

    grp1.create_dataset('ingroup1_1', data=data)
    grp1.create_dataset('ingroup1_2', data=data)
    fid.create_dataset('Group2/ingroup2', data=data)
    grp3.create_dataset('ingroup3', data=data)

    grp6.create_dataset('ingroup6', data=data)

    fid['base'].attrs['Attribute_str'] = 'Test'
    fid['base'].attrs['Attribute_bytes'] = b'Test'
    fid['base'].attrs['Attribute_np_bytes'] = np.bytes_('Test') # pylint: disable=no-member
    fid['base'].attrs.create('Attribute_int', 1)
    fid['base'].attrs.create('Attribute_float', 1.1)
    fid['base'].attrs.create('Attribute_np_1d', np.array([1, 2, 3]))
    fid['base'].attrs.create('Attribute_np_2d', np.array([[1, 2, 3], [4, 5, 6]]))

    yield filename, fid

    # Tear-down
    if hdf_is_open(fid):
        fid.close()
    os.remove(filename)

def test_get_groups(hdf_dataset):  # pylint:disable=redefined-outer-name
    """ Get an HDF5 file's group list """

    filename, fid = hdf_dataset

    # Passing fid
    grp_list = get_groups(fid)
    assert set(grp_list) == {'/', 'Group1', 'Group2', 'Group2/Group3', 'Group4', 'Group4/Group5',
                             'Group4/Group5/Group6'}

    # Passing filename
    grp_list = get_groups(filename)
    assert set(grp_list) == {'/', 'Group1', 'Group2', 'Group2/Group3', 'Group4', 'Group4/Group5',
                             'Group4/Group5/Group6'}

def test_get_datasets_fullpath(hdf_dataset):  # pylint:disable=redefined-outer-name
    """ Get an HDF5 file's dataset list with groupnames prepended"""
    filename, fid = hdf_dataset

    # Passing fid
    dataset_list = get_datasets(fid, fulldsetpath=True)
    assert set(dataset_list) == {'base', 'Group1/ingroup1_1', 'Group1/ingroup1_2',
                                 'Group2/ingroup2', 'Group2/Group3/ingroup3',
                                 'Group4/Group5/Group6/ingroup6'}

    # Passing filename
    dataset_list = get_datasets(filename, fulldsetpath=True)
    assert set(dataset_list) == {'base', 'Group1/ingroup1_1', 'Group1/ingroup1_2',
                                 'Group2/ingroup2', 'Group2/Group3/ingroup3',
                                 'Group4/Group5/Group6/ingroup6'}

def test_get_datasets_nopath(hdf_dataset):  # pylint:disable=redefined-outer-name
    """ Get an HDF5 file's dataset list with no groupnames prepended """
    filename, fid = hdf_dataset

    # Passing fid
    dataset_list = get_datasets(fid, fulldsetpath=False)
    assert set(dataset_list) == {'base', 'ingroup1_1', 'ingroup1_2', 'ingroup2',
                                 'ingroup3', 'ingroup6'}

    # Passing filename
    dataset_list = get_datasets(filename, fulldsetpath=False)
    assert set(dataset_list) == {'base', 'ingroup1_1', 'ingroup1_2', 'ingroup2',
                                 'ingroup3', 'ingroup6'}


def test_get_hierarchy_fullpath(hdf_dataset):  # pylint:disable=redefined-outer-name
    """
    OrderedDict describing HDF5 file hierarchy. Testing with full paths in
    the dataset names. Keys are groups, values are datasets.
    """
    filename, fid = hdf_dataset

    # Passing fid
    hierarchy = get_hierarchy(fid, fulldsetpath=True, grp_w_dset=False)
    assert hierarchy == {'/':['base'],
                         'Group1':['Group1/ingroup1_1', 'Group1/ingroup1_2'],
                         'Group2':['Group2/ingroup2'],
                         'Group2/Group3':['Group2/Group3/ingroup3'],
                         'Group4':[],
                         'Group4/Group5':[],
                         'Group4/Group5/Group6':['Group4/Group5/Group6/ingroup6']
                        }

    # Passing filename
    hierarchy = get_hierarchy(filename, fulldsetpath=True, grp_w_dset=False)
    assert hierarchy == {'/':['base'],
                         'Group1':['Group1/ingroup1_1', 'Group1/ingroup1_2'],
                         'Group2':['Group2/ingroup2'],
                         'Group2/Group3':['Group2/Group3/ingroup3'],
                         'Group4':[],
                         'Group4/Group5':[],
                         'Group4/Group5/Group6':['Group4/Group5/Group6/ingroup6']
                        }


def test_get_hierarchy_grp_w_dset(hdf_dataset):  # pylint:disable=redefined-outer-name
    """
    OrderedDict describing HDF5 file hierarchy. Testing empty sets are NOT
    returned. Keys are groups, values are datasets.
    """

    filename, fid = hdf_dataset

    # Passing fid
    hierarchy = get_hierarchy(fid, fulldsetpath=True, grp_w_dset=True)
    assert hierarchy == {'/':['base'],
                         'Group1':['Group1/ingroup1_1', 'Group1/ingroup1_2'],
                         'Group2':['Group2/ingroup2'],
                         'Group2/Group3':['Group2/Group3/ingroup3'],
                         'Group4/Group5/Group6':['Group4/Group5/Group6/ingroup6']
                        }

    # Passing filename
    hierarchy = get_hierarchy(filename, fulldsetpath=True, grp_w_dset=True)
    assert hierarchy == {'/':['base'],
                         'Group1':['Group1/ingroup1_1', 'Group1/ingroup1_2'],
                         'Group2':['Group2/ingroup2'],
                         'Group2/Group3':['Group2/Group3/ingroup3'],
                         'Group4/Group5/Group6':['Group4/Group5/Group6/ingroup6']
                        }

def test_get_hierarchy_nopath(hdf_dataset):  # pylint:disable=redefined-outer-name
    """
    OrderedDict describing HDF5 file hierarchy. Testing with no full paths in
    the dataset names. Keys are groups, values are datasets.
    """
    filename, fid = hdf_dataset

    # Passing fid
    hierarchy = get_hierarchy(fid, fulldsetpath=False, grp_w_dset=False)
    assert hierarchy == {'/':['base'],
                         'Group1':['ingroup1_1', 'ingroup1_2'],
                         'Group2':['ingroup2'],
                         'Group2/Group3':['ingroup3'],
                         'Group4':[],
                         'Group4/Group5':[],
                         'Group4/Group5/Group6':['ingroup6']
                        }


    # Passing filename
    hierarchy = get_hierarchy(filename, fulldsetpath=False, grp_w_dset=False)
    assert hierarchy == {'/':['base'],
                         'Group1':['ingroup1_1', 'ingroup1_2'],
                         'Group2':['ingroup2'],
                         'Group2/Group3':['ingroup3'],
                         'Group4':[],
                         'Group4/Group5':[],
                         'Group4/Group5/Group6':['ingroup6']
                        }

def test_get_dset_attrs(hdf_dataset):  # pylint:disable=redefined-outer-name
    """ Get an HDF5 file's dataset list with groupnames prepended"""

    filename, fid = hdf_dataset

    # Passing fid
    # DO NOT CONVERT-to-STR
    dset_attrs = get_attrs_dset(fid, 'base', convert_to_str=False)
    assert dset_attrs['Attribute_str'] == 'Test'
    assert dset_attrs['Attribute_bytes'] == b'Test'
    assert dset_attrs['Attribute_np_bytes'] == b'Test'
    assert dset_attrs['Attribute_int'] == 1
    assert dset_attrs['Attribute_float'] == 1.1
    assert np.allclose(dset_attrs['Attribute_np_1d'], np.array([1, 2, 3]))
    assert np.allclose(dset_attrs['Attribute_np_2d'], np.array([[1, 2, 3], [4, 5, 6]]))

    # DO CONVERT-to-STR
    dset_attrs = get_attrs_dset(fid, 'base', convert_to_str=True)
    assert dset_attrs['Attribute_str'] == 'Test'
    assert dset_attrs['Attribute_bytes'] == 'Test'
    assert dset_attrs['Attribute_np_bytes'] == 'Test'
    assert dset_attrs['Attribute_int'] == 1
    assert dset_attrs['Attribute_float'] == 1.1
    assert np.allclose(dset_attrs['Attribute_np_1d'], np.array([1, 2, 3]))
    assert np.allclose(dset_attrs['Attribute_np_2d'], np.array([[1, 2, 3], [4, 5, 6]]))

    # Passing filename
    # DO NOT CONVERT-to-STR
    dset_attrs = get_attrs_dset(filename, 'base', convert_to_str=False)
    assert dset_attrs['Attribute_str'] == 'Test'
    assert dset_attrs['Attribute_bytes'] == b'Test'
    assert dset_attrs['Attribute_np_bytes'] == b'Test'
    assert dset_attrs['Attribute_int'] == 1
    assert dset_attrs['Attribute_float'] == 1.1
    assert np.allclose(dset_attrs['Attribute_np_1d'], np.array([1, 2, 3]))
    assert np.allclose(dset_attrs['Attribute_np_2d'], np.array([[1, 2, 3], [4, 5, 6]]))

    # DO CONVERT-to-STR
    dset_attrs = get_attrs_dset(filename, 'base', convert_to_str=True)
    assert dset_attrs['Attribute_str'] == 'Test'
    assert dset_attrs['Attribute_bytes'] == 'Test'
    assert dset_attrs['Attribute_np_bytes'] == 'Test'
    assert dset_attrs['Attribute_int'] == 1
    assert dset_attrs['Attribute_float'] == 1.1
    assert np.allclose(dset_attrs['Attribute_np_1d'], np.array([1, 2, 3]))
    assert np.allclose(dset_attrs['Attribute_np_2d'], np.array([[1, 2, 3], [4, 5, 6]]))



""" Macros for inspection of HDF5 files """
from collections import OrderedDict as _OrderedDict

import h5py as _h5py
import numpy as _np

from lazy5.utils import (FidOrFile as _FidOrFile)

__all__ = ['get_groups', 'get_datasets', 'get_hierarchy',
           'get_attrs_dset']

def get_groups(file):
    """
    Parameters
    ----------

    file : str or h5py.File
        Filename or File-object for open HDF5 file

    Notes
    -----
    Gets groups in a hierarchical list starting from the base '/'. Thus if
    Group2 is INSIDE Group1, it will return Group1, Group1/Group2 -- NOT Group2
    inidividually.
    """
    # Get fid for a file (str or open fid)
    fof = _FidOrFile(file)
    fid = fof.fid

    all_items_list = []
    fid.visit(all_items_list.append)

    # list-set-list removes duplicates
    grp_list = list(set([item for item in all_items_list if isinstance(fid[item], _h5py.Group)]))

    grp_list.append('/')  # Add in base level group
    grp_list.sort()

    fof.close_if_file_not_fid()

    return grp_list

def get_datasets(file, fulldsetpath=True):
    """
    Parameters
    ----------

    file : str or _h5py.File
        Filename or File-object for open HDF5 file
    """
    # Get fid for a file (str or open fid)
    fof = _FidOrFile(file)
    fid = fof.fid

    all_items_list = []
    fid.visit(all_items_list.append)
    dset_list = []

    # list-set-list removes duplicates
    dset_list = list(set([item for item in all_items_list if isinstance(fid[item], _h5py.Dataset)]))
    dset_list.sort()

    if not fulldsetpath:
        for num, dset in enumerate(dset_list):
            split_out = dset.rsplit('/', maxsplit=1)
            if len(split_out) == 1:
                pass
            else:
                dset_list[num] = split_out[-1]

    fof.close_if_file_not_fid()

    return dset_list

def get_hierarchy(file, fulldsetpath=False, grp_w_dset=False):
    """
    Return an ordered dictionary, where the keys are groups and the items are
    the datasets

    Parameters
    ----------

    file : str or h5py.File
        Filename or File-object for open HDF5 file

    fulldsetpath : bool
        If True, a dataset name will be prepended with the group down to the
        base level, '/'. If False, it will just be the dset name.

    grp_w_dset : bool
        If True, only return groups that contain datasets. If False, include
        empty groups

    Returns
    -------
    OrderedDict : (group, [dataset list])
        Group and dataset names

    """

    # Get fid for a file (str or open fid)
    fof = _FidOrFile(file)
    fid = fof.fid

    grp_list = get_groups(fid)
    dset_list = get_datasets(fid, fulldsetpath=True)

    grp_dict = _OrderedDict([[grp, []] for grp in grp_list])

    for dset in dset_list:
        split_out = dset.rsplit('/', maxsplit=1)
        if len(split_out) == 1:
            grp_dict['/'].append(dset)
        else:
            if fulldsetpath:
                grp_dict[split_out[0]].append(dset)
            else:
                grp_dict[split_out[0]].append(split_out[1])

    # Only keep groups with datasets
    if grp_w_dset:
        to_pop = []
        for k in grp_dict:
            if not grp_dict[k]:  # is empty
                to_pop.append(k)

        for empty_grp in to_pop:
            grp_dict.pop(empty_grp)

    fof.close_if_file_not_fid()

    return grp_dict

def get_attrs_dset(file, dset, convert_to_str=True):
    """
    Get dictionary of attribute values for a given dataset

    Parameters
    ----------

    file : str or h5py.File
        Filename or File-object for open HDF5 file

    dset : str
        Full dataset name with preprended group names. E.g., '/Group1/Dataset'

    convert_to_str : bool
        If an attribute is a numpy.bytes_ string-like object, but not a str, try
        to decode into utf-8.

    Returns
    -------
    OrderedDict : (key, value)

    """

    # Get fid for a file (str or open fid)
    fof = _FidOrFile(file)
    fid = fof.fid

    ds_attrs = fid[dset].attrs

    attr_keys_list = list(ds_attrs)
    attr_keys_list.sort()

    attr_list = []
    for k in attr_keys_list:
        try:
            attr_val = ds_attrs[k]
        except (TypeError, ValueError):
            print('Could not get value for attribute: {}. Set to None'.format(k))
            attr_list.append([k, None])
        else:
            if (isinstance(attr_val, _np.bytes_) | isinstance(attr_val, bytes)) & convert_to_str: # pylint: disable=no-member
                attr_list.append([k, attr_val.decode()])
            else:
                attr_list.append([k, attr_val])

    attr_dict = _OrderedDict(attr_list)

    fof.close_if_file_not_fid()

    return attr_dict



""" Test non-HDF-related utilities """
import pytest

from lazy5.nonh5utils import filterlist

def test_filter_list():
    """ Test filtering of lists """
    list_to_filter = ['Keep1', 'Keep2', 'KeepExclude', 'Exclude1', 'Exclude2']

    # Keep, filter=str, exclusive
    filters = 'Keep'
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=True,
                          exclusive=True)
    assert out_list == ['Keep1', 'Keep2', 'KeepExclude']

    # Exclude, filter=str, exclusive
    filters = 'Exclude'
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=False,
                          exclusive=True)
    assert out_list == ['Keep1', 'Keep2']

    # Keep, filter=list, exclusive
    filters = ['Keep']
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=True,
                          exclusive=True)
    assert out_list == ['Keep1', 'Keep2', 'KeepExclude']

    # Keep, filter=tuple, exclusive
    filters = ('Keep')
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=True,
                          exclusive=True)
    assert out_list == ['Keep1', 'Keep2', 'KeepExclude']

    # Keep, filter=list, exclusive
    filters = ['Keep', '1']
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=True,
                          exclusive=True)
    assert out_list == ['Keep1']

    # Keep, filter=list, NOT-exclusive
    filters = ['Keep', '1']
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=True,
                          exclusive=False)
    assert out_list == ['Keep1', 'Keep2', 'KeepExclude', 'Exclude1']

    # Exclude, filter=list, exclusive
    filters = ['Exclude', '2']
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=False,
                          exclusive=True)
    assert out_list == ['Keep1']

    # Exclude, filter=list, NON-exclusive
    filters = ['Exclude', '2']
    out_list = filterlist(list_to_filter, filters, keep_filtered_items=False,
                          exclusive=False)

    assert out_list == ['Keep1', 'Keep2', 'KeepExclude', 'Exclude1']

    # Wrong type of filter
    filters = 1
    with pytest.raises(TypeError):
        out_list = filterlist(list_to_filter, filters, keep_filtered_items=False,
                              exclusive=False)
                              


""" Test inspection of HDF5 files """
import os
import sys
import h5py

import numpy as np
import pytest

try:
    from PyQt5.QtWidgets import QApplication
    from PyQt5.QtTest import QTest
    import PyQt5.QtCore
    from PyQt5.QtCore import Qt
except:
    HAS_PYQT5 = False
else:
    HAS_PYQT5 = True
    from lazy5.ui.QtHdfLoad import HdfLoad

from lazy5.utils import hdf_is_open

@pytest.mark.skipif(not HAS_PYQT5, reason='PyQt5 not installed, skipping.')
class TestUI:
    """ Test the HDF5 PyQt5 Viewer  """

    @pytest.fixture(scope="module")
    def hdf_dataset(self):
        """ Setups and tears down a sample HDF5 file """
        filename = 'temp_test_ui.h5'
        fid = h5py.File(filename, 'w')
        data_m, data_n, data_p = [20, 22, 24]
        data = np.random.randn(data_m, data_n, data_p)

        fid.create_dataset('base', data=data)

        grp1 = fid.create_group('Group1')
        grp3 = fid.create_group('Group2/Group3')
        grp6 = fid.create_group('Group4/Group5/Group6')

        grp1.create_dataset('ingroup1_1', data=data)
        grp1.create_dataset('ingroup1_2', data=data)
        fid.create_dataset('Group2/ingroup2', data=data)
        grp3.create_dataset('ingroup3', data=data)

        grp6.create_dataset('ingroup6', data=data)

        fid['base'].attrs['Attribute_str'] = 'Test'
        fid['base'].attrs['Attribute_bytes'] = b'Test'
        fid['base'].attrs['Attribute_np_bytes'] = np.bytes_('Test') # pylint: disable=no-member
        fid['base'].attrs.create('Attribute_int', 1)
        fid['base'].attrs.create('Attribute_float', 1.1)
        fid['base'].attrs.create('Attribute_np_1d', np.array([1, 2, 3]))
        fid['base'].attrs.create('Attribute_np_2d', np.array([[1, 2, 3], [4, 5, 6]]))

        app = QApplication(sys.argv)  # pylint: disable=C0103, W0612
        yield filename

        # Tear-down
        if hdf_is_open(fid):
            fid.close()
        os.remove(filename)
        # sys.exit()

    def test_ui_load_file(self, hdf_dataset):
        """ Load test file and check groups """
        self.filename = hdf_dataset
        dialog = HdfLoad()
        _ = dialog.fileOpen(self.filename)

        list_dsets = [dialog.ui.listDataSet.item(num).text() for num in
                      range(dialog.ui.listDataSet.count())]

        list_grps = [dialog.ui.comboBoxGroupSelect.itemText(num) for num in
                     range(dialog.ui.comboBoxGroupSelect.count())]

        assert list_dsets == ['base']
        assert 'Group1' in list_grps
        assert 'Group2/Group3' in list_grps
        assert 'Group4/Group5/Group6' in list_grps
        assert 'Group5' not in list_grps

    def test_ui_change_grp_and_filter_include(self, hdf_dataset):
        """ Load test file, change to Group1, filter for _1 """
        self.filename = hdf_dataset
        dialog = HdfLoad()
        _ = dialog.fileOpen(self.filename)

        # Change group to Group1
        dialog.ui.comboBoxGroupSelect.setCurrentIndex(1)
        list_dsets = [dialog.ui.listDataSet.item(num).text() for num in
                      range(dialog.ui.listDataSet.count())]
        assert dialog.ui.comboBoxGroupSelect.currentText() == 'Group1'
        assert list_dsets == ['ingroup1_1', 'ingroup1_2']

        dialog.ui.filterIncludeString.setText('_1')
        QTest.mouseClick(dialog.ui.pushButtonFilter, Qt.LeftButton)
        list_dsets = [dialog.ui.listDataSet.item(num).text() for num in
                      range(dialog.ui.listDataSet.count())]
        assert list_dsets == ['ingroup1_1']

    def test_ui_change_grp_and_filter_exclude(self, hdf_dataset):
        """ Load test file, change to Group1, filter for _1 """
        self.filename = hdf_dataset
        dialog = HdfLoad()
        _ = dialog.fileOpen(self.filename)

        # Change group to Group1
        dialog.ui.comboBoxGroupSelect.setCurrentIndex(1)
        list_dsets = [dialog.ui.listDataSet.item(num).text() for num in
                      range(dialog.ui.listDataSet.count())]
        assert dialog.ui.comboBoxGroupSelect.currentText() == 'Group1'
        assert list_dsets == ['ingroup1_1', 'ingroup1_2']

        dialog.ui.filterExcludeString.setText('_1')
        QTest.mouseClick(dialog.ui.pushButtonFilter, Qt.LeftButton)
        list_dsets = [dialog.ui.listDataSet.item(num).text() for num in
                      range(dialog.ui.listDataSet.count())]
        assert list_dsets == ['ingroup1_2']

    def test_ui_attrs(self, hdf_dataset):
        """ Load test file, change to base group (/), check attributes """
        self.filename = hdf_dataset
        dialog = HdfLoad()
        _ = dialog.fileOpen(self.filename)

        # Change group to Group1
        dialog.ui.comboBoxGroupSelect.setCurrentIndex(0)
        list_dsets = [dialog.ui.listDataSet.item(num).text() for num in
                      range(dialog.ui.listDataSet.count())]
        assert dialog.ui.comboBoxGroupSelect.currentText() == '/'
        assert list_dsets == ['base']

        # Select dataset base
        dialog.ui.listDataSet.item(0).setSelected(True)
        QTest.mouseClick(dialog.ui.listDataSet.viewport(), Qt.LeftButton)

        assert (dialog.ui.tableAttributes.findItems('Attribute_str', Qt.MatchExactly)[0].text() ==
                'Attribute_str')
        assert not dialog.ui.tableAttributes.findItems('fake', Qt.MatchExactly)  # Empty

    def test_ui_wrongfile(self, hdf_dataset):
        """ Load test file, change to base group (/), check attributes """
        self.filename = hdf_dataset
        dialog = HdfLoad()
        with pytest.raises(FileNotFoundError):
            _ = dialog.fileOpen('does_not_exist.h5')



"""
HDF5 LOAD DATA QDialog (crikit.vis.subguis.h5loadgui)
=======================================================

    H5LoadGUI : A graphical user interface (GUI) to select HDF5 dataset(s)

    Method : H5LoadGUI.getFileDataSets()

    Return (tuple) : (path [str], filename [str], dataset(s) [list], selection_made [bool])

    Notes
    -----
    Methods that interact with Qt follow the Qt naming convention:
    firstSecondThird
"""


# Append sys path
import sys as _sys
import os as _os

try:
    # Generic imports for QT-based programs
    from PyQt5.QtWidgets import (QApplication as _QApplication, \
    QDialog as _QDialog, QFileDialog as _QFileDialog, \
    QTableWidgetItem as _QTableWidgetItem)
except:
    HAS_PYQT5 = False
else:
    HAS_PYQT5 = True
from lazy5.ui.qt_HdfLoad import Ui_Dialog

from lazy5.inspect import get_hierarchy, get_attrs_dset
from lazy5.nonh5utils import filterlist

class HdfLoad(_QDialog): ### EDIT ###
    """ GUI Loader Class for H5 Files """

    # Default configuration
    config = {'only_show_grp_w_dset': True,  # Only show groups with datasets
              'attr_description': 'Memo',  # Description attribute key (optional)
              'excl_filtering' : True  # Filtering is exclusive (filters are AND'd)
             }

    def __init__(self, parent=None):

        # Generic load/init designer-based GUI
        super(HdfLoad, self).__init__(parent)
        self.ui = Ui_Dialog()  # pylint: disable=C0103
        self.ui.setupUi(self)

        self.path = None
        self.filename = None
        self.all_selected = None
        self.group_dset_dict = None

        self.ui.pushButtonOk.clicked.connect(self.accept)
        self.ui.pushButtonCancel.clicked.connect(self.reject)
        self.ui.comboBoxGroupSelect.currentTextChanged.connect(self.dataGroupChange)
        self.ui.listDataSet.itemClicked.connect(self.datasetSelected)
        self.ui.pushButtonFilter.clicked.connect(self.filterDatasets)
        self.ui.pushButtonResetFilter.clicked.connect(self.dataGroupChange)


    @staticmethod
    def getFileDataSets(pth='./', parent=None):  # pylint: disable=C0103; # pragma: no cover
        """
        Retrieve the filename and datasets selected by the user (via GUI)

        Parameters
        ----------
        pth : str
            Home directory to start in OR the relative pth to a file

        Returns
        ----------
        Tuple (str, str, list[str]) as (path, filename, [dataset(s)])

        """

        # pragma: no cover
        dialog = HdfLoad(parent)
        ret_fileopen = dialog.fileOpen(pth)

        ret = None
        if ret_fileopen:
            ret_dset_select = dialog.exec_()
            if ret_dset_select == _QDialog.Rejected:
                pass
            elif dialog.all_selected is None:
                pass
            else:
                ret = (dialog.path, dialog.filename, dialog.all_selected)
        return ret

    def fileOpen(self, pth='./'):  # Qt-related pylint: disable=C0103
        """ Select HDF5 File via QDialog built-in."""

        if _os.path.isdir(pth):  # No file provided, use QFileDialog; # pragma: no cover
            filetype_options = 'HDF5 Files (*.h5 *.hdf);;All Files (*.*)'
            full_pth_fname, _ = _QFileDialog.getOpenFileName(self, 'Open H5 File', pth,
                                                             filetype_options)
        elif _os.path.isfile(pth):  # Is a valid file
            full_pth_fname = pth
        else:
            raise FileNotFoundError('Not a valid path. Not a valid file.')

        ret = None
        if full_pth_fname:
            full_pth_fname = _os.path.abspath(full_pth_fname)  # Ensure correct /'s for each OS
            self.filename = _os.path.basename(full_pth_fname)
            self.path = _os.path.dirname(full_pth_fname)
            self.populateGroups()
            ret = True
        return ret

    def populateGroups(self):  # Qt-related pylint: disable=C0103
        """ Populate dropdown box of group ui.comboBoxGroupSelect """
        self.group_dset_dict = get_hierarchy(_os.path.join(self.path, self.filename),
                                             grp_w_dset=HdfLoad.config['only_show_grp_w_dset'])
        # Load Group dropdown box
        self.ui.comboBoxGroupSelect.clear()
        for count in self.group_dset_dict:
            self.ui.comboBoxGroupSelect.addItem(count)
        return [self.path, self.filename]

    def dataGroupChange(self):  # Qt-related pylint: disable=C0103
        """ Action : ComboBox containing Groups with DataSets has changed"""

        #self.dsetlist = QListWidget(self.verticalLayoutWidget)
        self.ui.listDataSet.clear()
        self.ui.listDataSet.addItems(self.group_dset_dict[self.ui.comboBoxGroupSelect.currentText()])
        #print('Changed')

    def populate_attrs(self, attr_dict=None):
        """ Populate attribute and memo boxes for currently selected dataset """

        self.ui.tableAttributes.setRowCount(0)
        self.ui.tableAttributes.setColumnCount(2)
        self.ui.tableAttributes.setSortingEnabled(False)
        self.ui.textDescription.setText('')

        if attr_dict:
            try:
                self.ui.textDescription.setText(attr_dict[HdfLoad.config['attr_description']])
            except (KeyError, AttributeError) as error_msg:
                print('{}\nNo memo at key {}'.format(error_msg, HdfLoad.config['attr_description']))

            for num, key in enumerate(attr_dict):
                self.ui.tableAttributes.insertRow(self.ui.tableAttributes.rowCount())
                self.ui.tableAttributes.setItem(num, 0, _QTableWidgetItem(key))
                self.ui.tableAttributes.setItem(num, 1, _QTableWidgetItem(str(attr_dict[key])))

    def datasetSelected(self):  # Qt-related pylint: disable=C0103
        """ Action : One or more DataSets were selected from the list """

        all_selected = self.ui.listDataSet.selectedItems()
        n_selected = len(all_selected)

        self.ui.textCurrentDataset.setText('')
        self.all_selected = []
        attrs = {}

        if n_selected > 0:
            current_selection = all_selected[-1].text()
            current_grp = self.ui.comboBoxGroupSelect.currentText()

            selection_str = '{} + ({} others)'.format(current_selection, n_selected - 1)
            self.ui.textCurrentDataset.setText(selection_str)
            current_dset_fullpath = '{}/{}'.format(current_grp, current_selection)
            attrs = get_attrs_dset(_os.path.join(self.path, self.filename),
                                   current_dset_fullpath, convert_to_str=True)
            self.all_selected = ['{}/{}'.format(current_grp, selection.text())
                                 for selection in all_selected]

        # Fill-in attribute table
        self.populate_attrs(attr_dict=attrs)

    def filterDatasets(self):  # Qt-related pylint: disable=C0103
        """ Filter list of datasets based on include and exclude strings """
        incl_str = self.ui.filterIncludeString.text()
        excl_str = self.ui.filterExcludeString.text()

        # From string with comma separation to list-of-strings
        incl_list = [q.strip() for q in incl_str.split(',') if q.strip()]
        excl_list = [q.strip() for q in excl_str.split(',') if q.strip()]

        dset_list = [self.ui.listDataSet.item(num).text() for num in
                     range(self.ui.listDataSet.count())]

        if incl_list:  # Include list is not empty
            dset_list = filterlist(dset_list, incl_list,
                                   keep_filtered_items=True,
                                   exclusive=HdfLoad.config['excl_filtering'])

        if excl_list:  # Exclude list is not empty
            dset_list = filterlist(dset_list, excl_list,
                                   keep_filtered_items=False,
                                   exclusive=HdfLoad.config['excl_filtering'])

        self.ui.listDataSet.clear()
        self.ui.listDataSet.addItems(dset_list)

if __name__ == '__main__':  # pragma: no cover
    app = _QApplication(_sys.argv)  # pylint: disable=C0103
    result = HdfLoad.getFileDataSets(pth='.')  # pylint: disable=C0103
    print('Result: {}'.format(result))

    _sys.exit()
    # pylint: error



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\ui_HdfLoad.ui'
#
# Created by: PyQt5 UI code generator 5.8.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(845, 588)
        Dialog.setStyleSheet("font: 10pt \"Arial\";")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout.setObjectName("verticalLayout")
        self.topHL = QtWidgets.QHBoxLayout()
        self.topHL.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.topHL.setObjectName("topHL")
        self.dataSetVL = QtWidgets.QVBoxLayout()
        self.dataSetVL.setObjectName("dataSetVL")
        self.label_1 = QtWidgets.QLabel(Dialog)
        self.label_1.setObjectName("label_1")
        self.dataSetVL.addWidget(self.label_1)
        self.comboBoxGroupSelect = QtWidgets.QComboBox(Dialog)
        self.comboBoxGroupSelect.setObjectName("comboBoxGroupSelect")
        self.dataSetVL.addWidget(self.comboBoxGroupSelect)
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.dataSetVL.addWidget(self.label_2)
        self.listDataSet = QtWidgets.QListWidget(Dialog)
        self.listDataSet.setEditTriggers(QtWidgets.QAbstractItemView.DoubleClicked|QtWidgets.QAbstractItemView.EditKeyPressed|QtWidgets.QAbstractItemView.SelectedClicked)
        self.listDataSet.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listDataSet.setObjectName("listDataSet")
        self.dataSetVL.addWidget(self.listDataSet)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.dataSetVL.addWidget(self.label_3)
        self.filterIncludeString = QtWidgets.QLineEdit(Dialog)
        self.filterIncludeString.setObjectName("filterIncludeString")
        self.dataSetVL.addWidget(self.filterIncludeString)
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setObjectName("label_4")
        self.dataSetVL.addWidget(self.label_4)
        self.filterExcludeString = QtWidgets.QLineEdit(Dialog)
        self.filterExcludeString.setObjectName("filterExcludeString")
        self.dataSetVL.addWidget(self.filterExcludeString)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButtonFilter = QtWidgets.QPushButton(Dialog)
        self.pushButtonFilter.setObjectName("pushButtonFilter")
        self.horizontalLayout.addWidget(self.pushButtonFilter)
        self.pushButtonResetFilter = QtWidgets.QPushButton(Dialog)
        self.pushButtonResetFilter.setObjectName("pushButtonResetFilter")
        self.horizontalLayout.addWidget(self.pushButtonResetFilter)
        self.dataSetVL.addLayout(self.horizontalLayout)
        self.topHL.addLayout(self.dataSetVL)
        self.attribVL = QtWidgets.QVBoxLayout()
        self.attribVL.setObjectName("attribVL")
        self.label_5 = QtWidgets.QLabel(Dialog)
        self.label_5.setObjectName("label_5")
        self.attribVL.addWidget(self.label_5)
        self.textCurrentDataset = QtWidgets.QTextBrowser(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textCurrentDataset.sizePolicy().hasHeightForWidth())
        self.textCurrentDataset.setSizePolicy(sizePolicy)
        self.textCurrentDataset.setMaximumSize(QtCore.QSize(16777215, 100))
        self.textCurrentDataset.setObjectName("textCurrentDataset")
        self.attribVL.addWidget(self.textCurrentDataset)
        self.label_6 = QtWidgets.QLabel(Dialog)
        self.label_6.setObjectName("label_6")
        self.attribVL.addWidget(self.label_6)
        self.tableAttributes = QtWidgets.QTableWidget(Dialog)
        self.tableAttributes.setStyleSheet("")
        self.tableAttributes.setColumnCount(2)
        self.tableAttributes.setObjectName("tableAttributes")
        self.tableAttributes.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        font = QtGui.QFont()
        font.setPointSize(10)
        item.setFont(font)
        item.setBackground(QtGui.QColor(121, 121, 121))
        brush = QtGui.QBrush(QtGui.QColor(91, 91, 91))
        brush.setStyle(QtCore.Qt.SolidPattern)
        item.setForeground(brush)
        self.tableAttributes.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        item.setBackground(QtGui.QColor(123, 123, 123))
        brush = QtGui.QBrush(QtGui.QColor(91, 91, 91))
        brush.setStyle(QtCore.Qt.SolidPattern)
        item.setForeground(brush)
        self.tableAttributes.setHorizontalHeaderItem(1, item)
        self.tableAttributes.horizontalHeader().setCascadingSectionResizes(True)
        self.tableAttributes.horizontalHeader().setDefaultSectionSize(200)
        self.tableAttributes.horizontalHeader().setHighlightSections(False)
        self.tableAttributes.horizontalHeader().setSortIndicatorShown(True)
        self.tableAttributes.horizontalHeader().setStretchLastSection(True)
        self.tableAttributes.verticalHeader().setVisible(False)
        self.tableAttributes.verticalHeader().setHighlightSections(False)
        self.attribVL.addWidget(self.tableAttributes)
        self.label_7 = QtWidgets.QLabel(Dialog)
        self.label_7.setObjectName("label_7")
        self.attribVL.addWidget(self.label_7)
        self.textDescription = QtWidgets.QTextBrowser(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textDescription.sizePolicy().hasHeightForWidth())
        self.textDescription.setSizePolicy(sizePolicy)
        self.textDescription.setMaximumSize(QtCore.QSize(16777215, 100))
        self.textDescription.setObjectName("textDescription")
        self.attribVL.addWidget(self.textDescription)
        self.topHL.addLayout(self.attribVL)
        self.verticalLayout.addLayout(self.topHL)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(-1, 10, -1, -1)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButtonOk = QtWidgets.QPushButton(Dialog)
        self.pushButtonOk.setObjectName("pushButtonOk")
        self.horizontalLayout_2.addWidget(self.pushButtonOk)
        self.pushButtonCancel = QtWidgets.QPushButton(Dialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout_2.addWidget(self.pushButtonCancel)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "HDF File Inspector"))
        self.label_1.setText(_translate("Dialog", "Groups Containing Datasets"))
        self.label_2.setText(_translate("Dialog", "Datasets"))
        self.label_3.setText(_translate("Dialog", "<html><head/><body><p><span style=\" font-weight:600;\">Include</span> Entires with Substring (separate by \',\' [comma])</p></body></html>"))
        self.label_4.setText(_translate("Dialog", "<html><head/><body><p><span style=\" font-weight:600;\">Exclude</span> Entires with Substring (separate by \',\' [comma])</p></body></html>"))
        self.pushButtonFilter.setText(_translate("Dialog", "Filter List"))
        self.pushButtonResetFilter.setText(_translate("Dialog", "Reset List"))
        self.label_5.setText(_translate("Dialog", "Current Selection"))
        self.label_6.setText(_translate("Dialog", "Atrribute Table"))
        self.tableAttributes.setSortingEnabled(True)
        item = self.tableAttributes.horizontalHeaderItem(0)
        item.setText(_translate("Dialog", "Attribute"))
        item = self.tableAttributes.horizontalHeaderItem(1)
        item.setText(_translate("Dialog", "Value"))
        self.label_7.setText(_translate("Dialog", "Dataset Information (Memo)"))
        self.pushButtonOk.setText(_translate("Dialog", "OK"))
        self.pushButtonCancel.setText(_translate("Dialog", "Cancel"))

